/*
 * File Name: pango_renderer.cpp
 */

/*
 * This file is part of uds-plugin-common.
 *
 * uds-plugin-common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * uds-plugin-common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Copyright (C) 2008 iRex Technologies B.V.
 * All rights reserved.
 */

#include "log.h"
#include "font_cache.h"
#include "pango_renderer.h"

int bmpwidth,bmpheight;
bool vertical=true; //switch to CJK Vertical Layout
bool render_set_vertical(unsigned int *width, unsigned int *height)
{
//vertical=true; //dummy now it is no function
//	if (*width==728) vertical=!vertical; // *width==984
//	return;
	bmpwidth=*width;   // remember the physical BMP buffer dimennsion
	bmpheight=*height;
	return true; 
/* for rotate BMP TO_DO_ 
	if (vertical) {
//		*width=bmpheight; 
//		*height=bmpwidth;
		
	}
	return vertical;*/
}

static int lineheight;
void render_set_lineheight(int line_height) 
{
	lineheight=line_height;
}

/* code from PSPSDK ccclib */
int cccUTF8toUCS2(unsigned int  * dst, int count, const char *str) {
	if (!str || !dst) return 0;

    int i = 0, length = 0;
    while (str[i] && length < count) {
		if  (str[i] <= 0x7FU) {       //ASCII
			dst[length] = (unsigned int )str[i]; 
			i++;    length++; 
		} else if (str[i] <= 0xC1U) { //part of multi-byte or overlong encoding ->ignore
			i++;          
		} else if (str[i] <= 0xDFU) { //2-byte
			dst[length] = ((str[i]&0x001fu)<<6) | (str[i+1]&0x003fu); 
			i += 2; length++; 
		} else if (str[i] <= 0xEFU) { //3-byte
			dst[length] = ((str[i]&0x001fu)<<12) | ((str[i+1]&0x003fu)<<6) | (str[i+2]&0x003fu); 
			i += 3; length++; 
		} else i++; //4-byte, restricted or invalid range ->ignore
	}
    return length;
}

/* for Chinese Vertical layout we must
   Things for Pango Layout 
   a) set the base_Gravity to PANGO_GRAVITY_EAST
   b) rotate 90
   Things At  Font data,
   I am not sure whether pango can do following things or not, especialy item (a). 
   a) unrotated the latin half glyphs
   b) unrotated the Chinese full vertical scripts 
   c) hinting some punctuations
   others are not considered, for example, time, digital index, combinational hanzi, 
   ------Spuggy
 */
#define zh_ROTCHAR 20 //vertical full for (b)
#define zh_PUNCHAR 15 //punctuations for (c)
//Rot　「」『』（）〔〕｛｝【】—…～《》〈〉＿
static unsigned int UCSLrotFullStr[zh_ROTCHAR]={
0x300C,0x300D,0x300E,0x300F,0xff08,0xff09,0x3014,0x3015,
0xff5b,0xff5d,0x3010,0x3011,0x2014,0x2026,0xff5e,0x300A,
0x300B,0x3008,0x3009,0xff3f};
//punctuations  ﹐﹑﹒﹔﹕﹖﹗
struct _PunHint{
	unsigned int ucscode;
	double       yfactor; // ratio of x to be move
	double       xfactor; // ratio of y to be move
}UCSLdotFullStrH[zh_PUNCHAR]={
//#define _UKAI_
//#define  _UMING_
#define  _WQY_ZENHEI_
#ifdef _UMING_
	{0xfe50, 0.0, -0.20},   //﹐   -0.25
	{0xfe51, 0.0, -0.185},    //﹑ -0.175
	{0xfe52, -0.0, -0.185},  //﹒    -0.175
	{0xfe54, 0.0, -0.000},   //﹔    -0.1
	{0xfe55, 0.0, -0.000},   //﹕    -0.10
	{0xfe56, 0.0, -0.000},   //﹖    -0.10
	{0xfe57, 0.0, -0.000},   //﹗    -0.10
	{0xff0c, -0.275, -0.4},	  //，   -0.25
	{0x3001, 0.0, 0},	  //、
	{0x3002, 0.0, 0},	  //。    
	{0xff1b, -0.35, -0.22},	  //；      -0.3 -0.2
	{0xff1a, -0.35, -0.22},	  //：
	{0xff01, -0.170, 0},	  //！   -.175  
	{0xff1f, -0.26, 0},	  //？     
	{0x2026, -0.25, 0} 	  //…  
#endif
#ifdef _WQY_ZENHEI_
	{0xfe50, 0.0, -0.20},   //﹐   -0.25
	{0xfe51, 0.0, -0.185},    //﹑ -0.175
	{0xfe52, -0.0, -0.185},  //﹒    -0.175
	{0xfe54, 0.0, -0.000},   //﹔    -0.1
	{0xfe55, 0.0, -0.000},   //﹕    -0.10
	{0xfe56, 0.0, -0.000},   //﹖    -0.10
	{0xfe57, 0.0, -0.000},   //﹗    -0.10
	
	{0xff0c, -0.0, -0.35},	  //，    
	{0x3001, -0.25, -0.35},	  //、
	{0x3002, -0.25, -0.35},	  //。    
	{0xff1b, -0.0, -0.185},	  //；      -0.3 -0.2
	{0xff1a, -0.0, -0.185},	  //：
	{0xff01, -0.0, 0},	  //！   -.175  
	{0xff1f, -0.0, 0},	  //？     
	{0x2026, -0.25, 0} 	  //…  
#endif
#ifdef _UKAI_
	{0xfe50, 0.0, -0.20},   //﹐   -0.25
	{0xfe51, 0.0, -0.185},    //﹑ -0.175
	{0xfe52, -0.0, -0.185},  //﹒    -0.175
	{0xfe54, 0.0, -0.000},   //﹔    -0.1
	{0xfe55, 0.0, -0.000},   //﹕    -0.10
	{0xfe56, 0.0, -0.000},   //﹖    -0.10
	{0xfe57, 0.0, -0.000},   //﹗    -0.10
	
	{0xff0c, -0.0, -0.00},	  //，    
	{0x3001, -0.00, -0.00},	  //、
	{0x3002, -0.00, -0.00},	  //。    
	{0xff1b, -0.0, -0.185},	  //；      -0.3 -0.2
	{0xff1a, -0.0, -0.185},	  //：
	{0xff01, -0.0, 0},	  //！   -.175  
	{0xff1f, -0.0, 0},	  //？     
	{0x2026, -0.25, 0} 	  //…  
#endif
};

static unsigned int  UCScode[512];
static bool latinflag;
void render_single_line(unsigned char   *bmp,
                        PangoLayoutLine *pango_line,
                        unsigned int    font_hash_code,
                        int             x,
                        int             y,
                        const int       width)
{
    // Draw run list .
    int glyph_pos_x = 0, glyph_pos_y = 0;
    GSList *runs_list = pango_line->runs;

    FontCache& font_cache = FontCache::instance();
    // draw each pango run
    while (runs_list)
    {
        // Get glyph data.
        PangoLayoutRun *run = (PangoLayoutRun *)runs_list->data;
        PangoItem        *item = run->item;
 		const char *text=pango_layout_get_text(pango_line->layout);
        PangoGlyphString *glyphs = run->glyphs;
		//PangoFont *font=run->item->analysis.font;
        FT_Face ft_face = pango_fc_font_lock_face(PANGO_FC_FONT(run->item->analysis.font));
		text+=item->offset;
 		/*int ucslen=*/cccUTF8toUCS2(UCScode,item->length ,text);
        // draw each glyph
        for (int i=0; i<glyphs->num_glyphs; i++)
        {
            if (glyphs->glyphs[i].glyph == PANGO_GLYPH_EMPTY)
            {
                continue;
            }
            PangoGlyphGeometry geometry = glyphs->glyphs[i].geometry;//glyphs->glyphs[i] is PangoGlyphInfo
// a) unrotated the latin half glyphs
// b) unrotated the Chinese full rot glyphs
			bool   rotcharFound=false;
			for(int j=0;j<zh_ROTCHAR;j++) // if vertical text don't rotated
				if (UCScode[i]==UCSLrotFullStr[j]){
					rotcharFound=true;
					break;
				}
			latinflag=(UCScode[i]<0x80)||rotcharFound;

		// hinting translation for Chinese font_family
            if (!latinflag&&vertical) { // ignore transform glyph translate vector
				x += geometry.width; 
            	glyph_pos_x = x + geometry.x_offset-geometry.width/8;
            	glyph_pos_y = y + geometry.y_offset+geometry.width/5; 
		    }else{ // rotated 90 degree Translate vector
            	glyph_pos_x = x + geometry.x_offset;
            	glyph_pos_y = y + geometry.y_offset;
				x += geometry.width; 
		    }
			// accroding the rotate 90 width the horizontal original, 
			// which is not the center of script, so we need to translate down
			if (vertical){
				glyph_pos_y+=lineheight*PANGO_SCALE/2; 
// c) hinting move the punctuations
				for(int j=0;j<zh_PUNCHAR;j++) // if vertical text don't rotated
					if (UCScode[i]==UCSLdotFullStrH[j].ucscode){
						rotcharFound=true; 
						glyph_pos_x+=UCSLdotFullStrH[j].xfactor*geometry.width; 
						glyph_pos_y+=UCSLdotFullStrH[j].yfactor*geometry.width; 
						break;
					}
				LOGPRINTF("(UDV)glyph at (%d,%d) [%2x] w=%d [%2x]\n", glyph_pos_x/PANGO_SCALE,glyph_pos_y/PANGO_SCALE,glyphs->glyphs[i].glyph,geometry.width,UCScode[i]);
			}
            // load glyph bitmap and copy
            FT_BitmapGlyph data = font_cache.get_glyph(ft_face,
                                                       font_hash_code,
                                                       glyphs->glyphs[i].glyph
													   ,!latinflag&&vertical); // tranform enable flag
            if (data)
            {
                render_single_glyph(bmp,
                                    PANGO_PIXELS(glyph_pos_x) + data->left,
                                    PANGO_PIXELS(glyph_pos_y) - data->top,
                                    width,
                                    data);
            }
        }

        pango_fc_font_unlock_face(PANGO_FC_FONT(run->item->analysis.font));
        runs_list = runs_list->next;
    }
}
#ifdef _WQY_ZENHEI_
void render_single_glyph(unsigned char        *bmp,
                         int                  x,
                         int                  y,
                         const int            width,
                         const FT_BitmapGlyph glyph)
{
    unsigned char *src = glyph->bitmap.buffer;
    unsigned char *dst = bmp + y * width + x;

    for(int i = 0; i < glyph->bitmap.rows; i++)
    {
        unsigned char *p = dst;
        for (int j = 0; j < glyph->bitmap.pitch; j++)
        {
			*p++^=*src++; 
        }
        dst += width; 
    }
}
#else
// try more thickness 
void render_single_glyph(unsigned char        *bmp,
                         int                  x,
                         int                  y,
                         const int            width,
                         const FT_BitmapGlyph glyph)
{
    unsigned char *src = glyph->bitmap.buffer;
    unsigned char *dst_02 = bmp + (y-2) * width + x;
    unsigned char *dst_0 = bmp + (y-1) * width + x;
    unsigned char *dst_1 = bmp + y * width + x;
    unsigned char *dst_2 = bmp + (y+1) * width + x;
	unsigned char center;

    for(int i = 0; i < glyph->bitmap.rows; i++)
    {
        unsigned char *p02 = dst_02;//mirror spuggy +width-1
        unsigned char *p0 = dst_0;//mirror spuggy +width-1
        unsigned char *p1 = dst_1;//mirror spuggy +width-1
        unsigned char *p2 = dst_2;//mirror spuggy +width-1
        for (int j = 0; j < glyph->bitmap.pitch; j++)
        {
			center=*src++; center^=0xff;
			*p1&=center; 
			if (!latinflag&&center!=0xff&&p02[-1]!=0xff&&p02[0]!=0xff&&p1[-2]!=0xff) { // more thickness &&((p0[-1]^p1[-1])!=0)&&((p0[0]^center)!=0)
			 	p0[-1]&=center; p0[0]&=center; // p0[1]&=center; 
				p1[-1]&=center; /*p1[0]^=center; p1[1]&=center; */
				//p2[-1]&=center; p2[0]&=center; p2[1]&=center; 
			}
			p02++; p0++; p1++; p2++; 
        }
        dst_0 += width; dst_1 += width; dst_2 += width;
    }
}
#endif
